function [dy,ddy]=multi_poly_diff(y,win,n)
% Moving window differentiator for real data
% Performs least squared fit of polynomial of degree n over windows of
% length win, finds analytical derivatives, evaluates at midpoint of window
% (except win/2 endpoints)

% INPUTS:
%   y: data to differentiate
%   win: window size (default is 11)
%   n: degree of polynomial (default is 2)

% OUTPUTS
%   dy: first derivative of y
%   ddy: second derivative of y

% check inputs
if nargin<3
    n=2;
end

if nargin<2
    win=11;
end

if length(y)<win
    error('Input y must be at least as long window')
end

% make window odd so point of interest is at center
if mod(win,2)~=1
    win=win+1;
end

wr=floor(win/2);
dn=length(y);
rsy=size(y);
y=reshape(y,[1,dn]);
dy=zeros(1,length(y));
ddy=zeros(1,length(y));

M = bsxfun(@power,(1:win)',0:n);
for i=1:dn-2*wr
    c= M\y(i:win+i-1)';
    for j=1:n
        dc(n-j+1)=c(j+1)*j;
        dy(i+wr)=dy(i+wr)+dc(n-j+1)*(wr+1)^(j-1);
    end
    for j=1:n-1
        ddc(n-j+1)=c(j+2)*j*(j+1);
        ddy(i+wr)=ddy(i+wr)+ddc(n-j+1)*(wr+1)^(j-1);
    end
    if i==1
        dc1=dc;
        ddc1=ddc;
    end
    if i==dn-2*wr
        dc2=dc;
        ddc2=ddc;
    end
end

dy(1:wr)=polyval(dc1,1:wr);
dy(end-wr:end)=polyval(dc2,win-wr:win);
ddy(1:wr)=polyval(ddc1,1:wr);
ddy(end-wr:end)=polyval(ddc2,win-wr:win);
dy=reshape(dy,rsy);
ddy=reshape(ddy,rsy);
end