function processVectorBinary(directory,cleanData)
% Reads in vector binary files and saves Vector.mat which has the
% following column structure:

%     1   timestamp                        serial date
%     2   Velocity (Beam1|X|East)          (m/s)
%     3   Velocity (Beam2|Y|North)         (m/s)
%     4   Velocity (Beam3|Z|Up)            (m/s)
%     5   Amplitude (Beam1)                (counts)
%     6   Amplitude (Beam2)                (counts)
%     7   Amplitude (Beam3)                (counts)
%     8   SNR (Beam1)                      (dB)
%     9   SNR (Beam2)                      (dB)
%     10  SNR (Beam3)                      (dB)
%     11  Correlation (Beam1)              (%)
%     12  Correlation (Beam2)              (%)
%     13  Correlation (Beam3)              (%)
%     14  Pressure                         (dbar)

% INPUTS (all optional)
% directory: indicates file path to vector binary files.
% cleanData=1: despikes the velocity data and cleans based on correlation

% Ben Strom, last edit, Oct. 1, 2015
if ~exist('directory','var')
    directory='';
end

if ~isempty(directory)
    currentFolder=pwd;
    cd(directory)
end

if ~exist('cleanData','var')
    cleanData=0;
end

if isempty(dir('vector.dat'))
    error('Can''t find vector.dat file. Have you convertedthe .vec file?')
end

if ~exist('dist','var')
    dist=[];
end

% Get sampling frequency
fid = fopen('vector.hdr', 'r');
C = textscan(fid, '%s', 'Delimiter', '\n');
fclose(fid);
freq_str=C{1,1}(12);
freq_str=freq_str{1};
sf=str2double(freq_str(39:40));
speriod=1/sf;

% Get start and end times
sen=importdata('vector.sen');
% Start time is one second after first vector.sen time

sen_start=datenum([sen(2,3),sen(2,1),sen(2,2),sen(2,4:6)]);
sen_end=datenum([sen(end,3),sen(end,1),sen(end,2),sen(end,4:6)]);

% Import data
data=importdata('vector.dat');
% Calculate coresponding timestamps
stamp_start=sen_start+speriod/2/(3600*24);
stamp_end=stamp_start+((length(data)-1)*speriod)/(3600*24);
time_stamps=stamp_start:speriod/(3600*24):stamp_end;

if cleanData==1
    % Get the trasnform matrix
    a=strfind(C{1,1},'Transformation matrix');
    tmind=find(~cellfun(@isempty,a));
    t1=C{1,1}(tmind);
    t1=sscanf(t1{1},'Transformation matrix                 %f %f %f')';
    t2=C{1,1}(tmind+1);
    t2=sscanf(t2{1},'%f %f %f')';
    t3=C{1,1}(tmind+2);
    t3=sscanf(t3{1},'%f %f %f')';
    T=[t1;t2;t3];
    U=[data(:,3), data(:,4), data(:,5)];
    % Transpose to beam
    Ubeam = (T\U')';
    
    % Find spikes to remove
    cutoffFreq=3; % High pass filter cuttoff in Hz. May need to adjust
    badSpike=zeros(length(Ubeam(:,1)),1);
    dif = designfilt('highpassfir', 'FilterOrder', 20, 'CutoffFrequency', cutoffFreq, 'SampleRate', sf);
    for i=1:3
        f=filtfilt(dif,detrend(double(Ubeam(:,i))));
        [~,spikeInds]=func_despike_phasespace3d(f);
        badSpike(spikeInds)=1;
    end
    
    % Find bad correlation locations
    cor=data(:,12:14);
    corLogic=ones(size(cor));
    corLogic(cor<70)=0;
    corLogic2=prod(corLogic,2);
    badCor=zeros(length(corLogic2),1);
    if sum(corLogic2==0)<0.1*length(corLogic2)
        % Then chuck the ones less than 70
        badCor(corLogic2==0,:)=1;
    else
        % Chuck the worst 10%
        [~,inds]=sort(cor);
        inds=reshape(inds',numel(inds),1);
        uniqueInds=unique(inds,'stable');
        chuckInds=uniqueInds(1:round(length(corLogic2)*0.1));
        badCor(chuckInds)=1;
    end
    
    % Merge correlation and despike
    bad=zeros(size(badSpike));
    bad(badSpike | badCor)=1;
    bad=logical(bad);
    % If needed, interpolate/remove points. Otherwise NaN
    inds=find(~bad);
    for i=1:3
        Ubeam(:,i)=interp1(inds,Ubeam(~bad,i),1:length(Ubeam(:,i)),'PCHIP');
    end
    
    data(:,3:5)=(T*Ubeam')';
end

% adjust timestamps based on velocity magnitude and distance to turbine
% if ~isempty(dist)
%     % Finish this!! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     U=sqrt(data(:,2).^2+data(:,3).^2+data(:,4).^2);
%     % smooth U using windowed mean
%     toffset=dist./U;
%     % time_stamps
% end

Vector=[time_stamps',data(:,3:15)];

if ~isempty(directory)
    cd( currentFolder)
end
save('Vector','Vector');

end