function v3CalcPointData(directory,trim,dspkVector,MomentOfI,turbsize)
% Computes mean and standard deviation of quantities of interest for every
% file set in 'directory' or in the current directory if none is given.
% if trim=1, (optional, default is trim=0, no trim) trims data to an even 
% number of turbine rotations before performing calculations
% To trim without using directory arg., use v3CalcPointData([],1)
% if turbsize [R,H] specified, then use these values, otherwise use lab
% standard defaults



%% Setup

curDir=pwd;
% change directory if given as argument
if exist('directory','var') && ~isempty(directory)
    cd(directory)
end

% check if trim is given, otherwise, set to zero
if ~exist('trim','var') || isempty(trim)
    trim=0;
end

if ~exist('dspkVector','var') || isempty(dspkVector)
    dspkVector=0;
end

if ~exist('turbsize','var') || isempty(turbsize)
    turbsize = [17.2/100, 23.4/100];
end

if ~exist('MomentOfI','var') || isempty(MomentOfI)
    MomentOfI=[];
end

%% Main code

% Get simulink data save files
[modelFileNames,modelFileIDs]=getFileNameIDs('modelData_*.mat');
if isempty(modelFileNames)
    error('No v3 data points detected in the current directory')
end
% Get timeIDs from file names

% Hardware config
if isempty(dir('HWconfig.mat'))
    curdir=pwd;
    cd('../')
    if isempty(dir('HWconfig.mat'))
        error('Can''t find HWconfig file')
    end
    load('HWconfig.mat')
    cd(curdir)
else
    load('HWconfig.mat')
end
    

% Perform AMTI file checks
if any(strcmp({HWconfig.inst},'AMTI top cell (digital)'))
    [AMTIFileNames,AMTIFileIDs]=getFileNameIDs('modelData_*.mat');
    % Check that the files exist
    if isempty(AMTIFileNames)
        error('AMTI cell indicated in HWconfig, but no AMTIData files found')
    end
    % Check that they match the modelData files
    if ~all(AMTIFileIDs==modelFileIDs) || length(AMTIFileIDs)~=length(modelFileIDs)
        error('modelData file ID num(s) do not match AMTIData file ID num(s)')
    end
end

% Perform Vectrino file checks
if any(strcmp({HWconfig.inst},'Vectrino (digital)'))
    [vectrino1Files,vectrino1FileIDs]=getFileNameIDs('vectrinoData1_*.mat');
    [vectrino2Files,vectrino2FileIDs]=getFileNameIDs('vectrinoData2_*.mat');
    if isempty(vectrino1Files) || isempty(vectrino2Files)
        error('Vectrinos indicated in HWconfig, but did not find 2 sets of Vectrino files.')
    end
    if length(vectrino1FileIDs)~=length(vectrino2FileIDs)
        error('The number of files found for each Vectrino is not the same.')
    end
    if ~all(vectrino1FileIDs==modelFileIDs) || ~all(vectrino2FileIDs==modelFileIDs)
        error('modelData file ID num(s) do not match VectrinoData file ID num(s).')
    end
end


for i=1:length(modelFileIDs)
    fprintf('Processing point %.0f of %.0f.\n',i,length(modelFileIDs))
    td(i)=v3PostProcess(modelFileIDs(i),trim,dspkVector,MomentOfI); 
    
    p.MZ(i,1)=mean(td(i).MZ); 
    p.FZ(i)=mean(td(i).FZ);
    % check for fields, then calculate other mean forces
    p.turbVel(i)=mean(td(i).turbVel);
    
    p.power(i)=mean(td(i).turbVel.*td(i).MZ);
 

    perf.turbVelStd(i)=std(td(i).turbVel);

    perf.turbTorque(i)=mean(td(i).MZ);
    
    if any(strcmp({HWconfig.inst},'Vector (digital)'))
        u=td(i).digitalVector(:,1); v=td(i).digitalVector(:,2); w=td(i).digitalVector(:,3);
        Uinf=sqrt(u.^2+v.^2+w.^2);
        p.Uinf(i)=mean(Uinf);
    
         % Turbulence Intensity
        p.TI(i)=mean(sqrt(1/3*((u-mean(u)).^2+(v-mean(v)).^2+(w-mean(w)).^2))./Uinf);
         p.TSR(i)=p.turbVel(i)*(turbsize(1)/2)/p.Uinf(i);
            p.eff(i)=p.power(i)/(.5*1000*mean(Uinf.^3)*turbsize(1)*turbsize(2));
    end
    
    if  any(strcmp({HWconfig.inst},'Temperature probe'))
        p.temp(i)=mean(td(i).temp); %flume temp deg C
    end
    

    %calculate tip speed ratio, mechanical power, and efficiency



    % DOM, you may have to make sure these are correct %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % p.Egen(i)=trapz(td(i).turbVel.*td(i).MZ)./2000; %Fs= 2000
    %for Bamfield let the temperature be constant
   % p.CpMax(i)=-4.4835e-4*(p.temp(i)^2) + 0.0286*p.temp(i) - 0.2994;
    %p.CpMax(i)=0.14;
    %p.Emax(i)=trapz(0.5*.0402.*1000.*(Uinf.^3).*p.CpMax(i))./32;
    %p.Eloss(i)=(1-(p.Egen(i)./p.Emax(i))).*100; % as a percentage
    
    % clear reused variables
    Uinf=[]; u=[]; v=[]; w=[];
end

save('pointData','p','td')

cd(curDir)