function v3PhaseAverage(N,bins,turbineIsConstant,td)
% Calculates phase averaged quantities of interest
% N: number of turbine blades - phase averages over N segments per rotation
% bins: Number of bins to phase average over.
% If turbineIsConstant=1 (optional), the phase averaged turbine performance
% data and forces are averaged over all data points (use if the turbine was
% at constant settings - like for an ADV wake sweep)

% Ben Strom, Aug. 18, 2015
density=1000;
turbH=0.234; % turbine Height
turbD=0.172; % turbine Diameter

if ~exist('turbineIsConstant','var')
    turbineIsConstant=0;
else
    Y1all=[];
    Y2all=[];
    turbPosAll=[];
end



if  ~exist('td','var')
    if isempty(dir('pointData.mat'))
        delete('Vector.mat');
        v3CalcPointData([],1,1);
    end
    load('pointData.mat')
end

load('HWconfig')

if isempty(dir('turbConfig.mat'))
%     warning('No turbConfig file found.  Using preset pitch angle = 6 degrees')
    presetPitch = 6;
else
    load('turbConfig.mat')
    presetPitch=turbConfig.pAngle;
end

for i=1:length(td)
    % td(i).turbPos=(td(i).turbPos-1.3090314446).*4-6.0421499e-11+4.974637e-16+9.3988e-22+3.122301e-27-3.3487e-33-2e-38-3.6595e-39;
    % These fields should always exist, so are processed all at once
    instPower=td(i).MZ.*td(i).turbVel;
    % Free stream magnitude.
    if any(strcmp({HWconfig.inst},'Vector (digital)'))
        fsU=sqrt(td(i).digitalVector(:,1).^2+td(i).digitalVector(:,2).^2+td(i).digitalVector(:,3).^2);
    end
    % TSR and instPower are calculated differently depending on how the
    % Vector data has been processed.
    if length(instPower)~=length(fsU)
%         warning('Digital vector timeseries has not been interpolated. Using mean(U^3) in instantaneous efficiency calculation, and mean(U) in the instantaneous TSR calculation.')
        instEff=instPower./(0.5*density*mean(fsU.^3)*turbD*turbH);
        TSR=td(i).turbVel.*turbD/(2*mean(fsU));
        CQ=td(i).MZ./(0.5*density*mean(fsU.^2)*turbD*turbH*turbD/2);
    else
        instEff=instPower./(0.5*1000*fsU.^3*0.172*0.234);
        TSR=td(i).turbVel.*0.172./(2.*fsU);
        CQ=td(i).MZ./(0.5*density.*fsU.^2.*turbD*turbH*turbD/2);
    end
    % Effective/nominal angle of attack
    alphaN=atan2(-sin(td(i).turbPos),(TSR+cos(td(i).turbPos)))+presetPitch*pi/180;
    % Matrix of values to phase average
    Y1=[td(i).turbVel,td(i).turbAcc,td(i).MZ,instPower,instEff,TSR,alphaN,CQ];
    
    if isfield(td(i),'MZF')
        instPowerF=td(i).MZF.*td(i).turbVel;
        instEffF=instPowerF./(0.5*density*mean(fsU.^3)*turbD*turbH);
        CQF = td(i).MZF./(0.5*density.*mean(fsU.^2).*turbD*turbH*turbD/2);
        Y1=[Y1,td(i).MZF,instPowerF,instEffF,CQF];
    end
    
    if isfield(td(i),'FX') && isfield(td(i),'FY') && isfield(td(i),'FZ')
        CL=td(i).FY./(0.5*density.*mean(fsU.^2).*turbD*turbH);
        CD=td(i).FX./(0.5*density.*mean(fsU.^2).*turbD*turbH);
        CA=td(i).FZ./(0.5*density.*mean(fsU.^2).*turbD*turbH);
        Y2=[td(i).FX,td(i).FY,td(i).FZ,CL,CD,CA];
    end
    
    if ~turbineIsConstant % then phase average this point, store
        [Yout,pa(i).binThetas,stdevs]=anglePhaseAverage(td(i).turbPos,Y1,bins,N);
        pa(i).turbVel=[Yout(:,1),stdevs(:,1)];
        pa(i).turbAcc=[Yout(:,2),stdevs(:,2)];
        pa(i).MZ=[Yout(:,3),stdevs(:,3)];
        pa(i).power=[Yout(:,4),stdevs(:,4)];
        pa(i).eff=[Yout(:,5),stdevs(:,5)];
        pa(i).TSR=[Yout(:,6),stdevs(:,6)];
        pa(i).alphaN=[Yout(:,7),stdevs(:,7)];
        pa(i).CQ=[Yout(:,8),stdevs(:,8)];
        
        if isfield(td(i),'MZF')
            pa(i).MZF=[Yout(:,9),stdevs(:,9)];
            pa(i).instPowerF=[Yout(:,10),stdevs(:,10)];
            pa(i).instEffF=[Yout(:,11),stdevs(:,11)];
            pa(i).CQF = [Yout(:,12),stdevs(:,12)];
        end
        clear Y stdevs Yout
        
        % Process fields that may or may not exist, depending on HWconfig
        if isfield(td(i),'FX') && isfield(td(i),'FY') && isfield(td(i),'FZ')
            [Yout,~,stdevs]=anglePhaseAverage(td(i).turbPos,Y2,bins,N);
            pa(i).FX=[Yout(:,1),stdevs(:,1)];
            pa(i).FY=[Yout(:,2),stdevs(:,2)];
            pa(i).FZ=[Yout(:,3),stdevs(:,3)];
            pa(i).CL=[Yout(:,4),stdevs(:,4)];
            pa(i).CD=[Yout(:,5),stdevs(:,5)];
            pa(i).CA=[Yout(:,6),stdevs(:,6)];
            clear Y stdevs Yout
        end
    else % Concatenate all data for later phase averaging if turbine is constant
        turbPosAll=[turbPosAll;td(i).turbPos];
        Y1all=[Y1all;Y1];
        if isfield(td(i),'FX') && isfield(td(i),'FY') && isfield(td(i),'FZ')
            Y2all=[Y2all;Y2];
        end
    end
    
    % Process wake data, if available
    if any(strcmp({HWconfig.inst},'Vectrino (digital)'))
        % These are the locations of the "good" values - logical (despike, correlation)
        goodInds1=~td(i).v0.badInds;
        goodInds2=~td(i).v1.badInds;
        
        % Good velocities
        U1=td(i).v0.U(goodInds1,:);
        U2=td(i).v1.U(goodInds2,:);
        % Perturbation velocities
        U1p=bsxfun(@minus,U1,mean(U1,1));
        U2p=bsxfun(@minus,U2,mean(U2,1));
        % Perterbation velocities squared
        U1p2=bsxfun(@minus,U1,mean(U1,1)).^2;
        U2p2=bsxfun(@minus,U2,mean(U2,1)).^2;
        
        
        % Cross terms of Reynolds stress
        RsCross1=[U1p(:,1).*U1p(:,2),U1p(:,1).*U1p(:,3),U1p(:,3).*U1p(:,2)];
        RsCross2=[U2p(:,1).*U2p(:,2),U2p(:,1).*U2p(:,3),U2p(:,3).*U2p(:,2)];
        % Turbulent intensity
        TI1=sqrt(1/3.*(sum(U1p.^2,2)))./mean(sqrt(sum(U1.^2,2)));
        TI2=sqrt(1/3.*(sum(U2p.^2,2)))./mean(sqrt(sum(U2.^2,2)));
        
        % Position of turbine at good vectrino data points
        turbPos=td(i).turbPos(td(i).v0.sInd); % pos at every vel point
        turbPos1=turbPos(goodInds1); % Pos at good vel points
        turbPos2=turbPos(goodInds2);
        
        % Phase average
        Y1=[U1 U1p RsCross1 TI1 U1p2];
        Y2=[U2 U2p RsCross2 TI2 U2p2];
        [Yout1,binThetas,stdevs1]=anglePhaseAverage(turbPos1,Y1,bins,N);
        [Yout2,binThetas,stdevs2]=anglePhaseAverage(turbPos2,Y2,bins,N);
        paWake(i).U=[Yout1(:,1:3),stdevs1(:,1:3)];
        paWake(i+length(td)).U=[Yout2(:,1:3),stdevs2(:,1:3)];
        paWake(i).Up=[Yout1(:,4:6),stdevs1(:,4:6)];
        paWake(i+length(td)).Up=[Yout2(:,4:6),stdevs2(:,4:6)];
        paWake(i).RsCross=[Yout1(:,7:9),stdevs1(:,7:9)];
        paWake(i+length(td)).RsCross=[Yout2(:,7:9),stdevs2(:,7:9)];
        paWake(i).TI=[Yout1(:,10),stdevs1(:,10)];
        paWake(i+length(td)).TI=[Yout2(:,10),stdevs2(:,10)];
        paWake(i).Up2=[Yout1(:,11:13),stdevs1(:,11:13)];
        paWake(i+length(td)).Up2=[Yout2(:,11:13),stdevs2(:,11:13)];
        paWake(i).pos=td(i).v0.pos;
        paWake(i+length(td)).pos=td(i).v1.pos;
        if i==1
            paWake(i).binThetas=binThetas;
        end
        clear Y1 Y2 Yout1 Yout2 stdevs1 stdevs2
        
        % Mean Values (and standard deviations over time)
        mWake(i).U=[mean(U1,1) std(U1,1)];
        mWake(i+length(td)).U=[mean(U2,1) std(U2,1)];
        mWake(i).Up=[mean(U1p,1) std(U1p,1)];
        mWake(i+length(td)).Up=[mean(U2p,1) std(U2p,1)];
        mWake(i).Up2=[mean(U1p2,1) std(U1p2,1)];
        mWake(i+length(td)).Up2=[mean(U2p2,1) std(U2p2,1)];
        mWake(i).RsCross=[mean(RsCross1,1) std(RsCross1,1)];
        mWake(i+length(td)).RsCross=[mean(RsCross2,1) std(RsCross2,1)];
        mWake(i).TI=[mean(TI1) std(TI1)];
        mWake(i+length(td)).TI=[mean(TI2) std(TI2)];
        mWake(i).pos=td(i).v0.pos;
        mWake(i+length(td)).pos=td(i).v1.pos;
    end
end

% Phase average concatenated turbine data if turbine is constant
if turbineIsConstant
    [Yout,pa.binThetas,stdevs]=anglePhaseAverage(turbPosAll,Y1all,bins,N);
    pa.turbVel=[Yout(:,1),stdevs(:,1)];
    pa.turbAcc=[Yout(:,2),stdevs(:,2)];
    pa.MZ=[Yout(:,3),stdevs(:,3)];
    pa.power=[Yout(:,4),stdevs(:,4)];
    pa.eff=[Yout(:,5),stdevs(:,5)];
    pa.TSR=[Yout(:,6),stdevs(:,6)];
    pa.alphaN=[Yout(:,7),stdevs(:,7)];
    pa.CQ=[Yout(:,8),stdevs(:,8)];
    pa.CQF = [Yout(:,8),stdevs(:,8)];
    
    if isfield(td(1),'MZF')
        pa.MZF=[Yout(:,9),stdevs(:,9)];
        pa.instPowerF=[Yout(:,10),stdevs(:,10)];
        pa.instEffF=[Yout(:,11),stdevs(:,11)];
        pa.CQF=[Yout(:,11),stdevs(:,11)];
    end
    
    if isfield(td(i),'FX') && isfield(td(i),'FY') && isfield(td(i),'FZ')
        [Yout,~,stdevs]=anglePhaseAverage(turbPosAll,Y2all,bins,N);
        pa.FX=[Yout(:,1),stdevs(:,1)];
        pa.FY=[Yout(:,2),stdevs(:,2)];
        pa.FZ=[Yout(:,3),stdevs(:,3)];
        pa.CL=[Yout(:,4),stdevs(:,4)];
        pa.CD=[Yout(:,5),stdevs(:,5)];
        pa.CA=[Yout(:,6),stdevs(:,6)];
    end
end

saveVar.pa=pa;
clear pa

if exist('paWake','var')
    saveVar.paWake=paWake;
    clear paWake
    saveVar.mWake=mWake;
    clear mWake
end
save('phaseAvData','-struct','saveVar','-v7.3')


end