function td=v3PostProcess(timeID,trim,dspkVector,MomentOfI)
% Post process one data point (identified by integer timeID, which is time time the
% aquisition was initiated, measured in seconds after 1/1/2015 00:00:00)

% trim=1 (optional) trims the data to encompass an integer number of turbine
% revolutions. Trims from the front such that the first position is the
% same as the last. CHANGE: trim now first checks to see if encoder Z
% (index) has been registered , and if not, first trims the begining of the
% dataset to the first index mark.  Useful for data collected without the
% simulink model

% dspkVector=1 (optional) depikes the vector data during creation of the
% Vector mat file.

% MomentOfI can either be thge moment of inertia for the turbine or a
% system definition (like a transfer function). This is used to calculated
% the non-fluid forcing

% TO DO:
%    Check temperature probe callibration
%    Check command torque calibration
%    Free surface height transducers module

% Always fields of output structure td
%     * Turbine encoder data
%          - td.turbPos
%          - td.turbVel
%          - td.turbAcc
%     * Turbine command/target signals (both are always saved)
%          - td.commandVel
%          - td.commandTorque

% Possible additional fields of output structure td
%     * Forces on turbine:
%          - td.FX           Drag force
%          - td.MX           Drag Moment (w/out bottom loadcell only)
%          - td.FY           Lateral Force - to left if facing downstream
%          - td.MY           Lateral Moment (w/out bottom loadcell only)
%          - td.MZ           Torque applied to turbine in CCW direction
%          - td.FZ           Vertical force (up=positive)
%   If there are two load cells:
%     * Isolated AMTI top cell forces
%          - td.AMTIdata
%     * Isolated ATI bottom cell forces
%          - td.ATIdata
%   If instrument is present
%     * Analog AMTI torque output
%          - td.AMTIanalogTorque
%     * Temperature probe
%          - td.temperature
%     * Vector analog velocities
%          - td.analogVector
%     * Vectrino digital info structures
%          - td.v0
%          - td.v1
%          - td.v0/v1.sInd     Indicates which model data sample each Vectrino
%                           sample corresponds to
% Ben Strom, last edit, Sept. 29, 2015

%% Initialization

% if no trim option is given, do not trim
if ~exist('trim','var') || isempty(trim)
    trim=0;
end

% If no vector despike option is given, don't despike
if ~exist('dspkVector','var') ||  isempty(dspkVector)
    dspkVector=0;
end

% Check for tare file, if it doesn't exist, make it. Load it
tareFile=dir('tareFile.mat');
if isempty(tareFile)
    curDir = pwd;
    cd('../')
    if isempty(dir('tareFile'))
        makeTareFileSwitch = 1;
    else
        makeTareFileSwitch = 0;
        load('tareFile.mat')
    end
    cd(curDir)
else
   load('tareFile.mat')
   makeTareFileSwitch = 0;
end

if makeTareFileSwitch
    if isempty(dir('forceTare*'))
        % check above
        curDir = pwd;
        cd('../')
        if isempty(dir('forceTare*'))
            error('Cannot Locate Tare Data Files')
        else
            if isempty(dir('tareFile.mat'))
                makeTareFile;
            end
        end
        load('tareFile.mat')
        cd(curDir)
    else
        if isempty('tareFile.mat')
            makeTareFile;
        end
        load('tareFile.mat')
    end
end



% Load in HW config. file
if ~isempty(dir('HWconfig.mat'))
    load('HWconfig.mat')
else
    curDir = pwd;
    cd('../')
    if isempty(dir('HWconfig.mat'))
        error('Cannot Locate HWconfig file')
    else
        load('HWconfig.mat')
    end
    cd(curDir);
end

%% Load in model data (DAQ and command signals)

modelDataFile=dir(strcat('modelData_',num2str(timeID),'.mat'));
if isempty(modelDataFile)
    error('No modelData file found with the given timeID num')
end

% Load in DAQ/model outputs file (load in data matrix "d" and structure
% "runInfo")
load(modelDataFile.name);
numDataPoints=length(d);
%% Calculate the trim index, if user asks for trim
% column of position index (1 vs 2 DAQs)
if size(d,2)==11
    posCol=9;
else
    posCol=17;
end

% First, check for single fuckup spots in the position trace. To see what
% this fixes, see point 20647501 in Bamfield_2015\wake_transects\P_1\tr_002
inds=find(d(:,posCol)<-2^16+1);
for i=2:length(inds)
    if abs(d(inds(i)-1,posCol))<2^16+1 && abs(d(inds(i)+1,posCol))<2^16
        np=2*d(inds(i)-1,posCol)-d(inds(i)-2,posCol);
        if np>-2^16
            d(inds(i),posCol)=np;
        else
            np=-2*d(inds(i)+1,posCol)+d(inds(i)+2,posCol);
            d(inds(i),posCol)=np;
        end
    end
end

if trim
    % First, determine whether encoder index pulse has been registered or if
    % it is registered during beginning of aquisition
    
    % Max position count magnitud in last quarter of aqusition
    maxPos=max(abs(d(round(size(d,1)*0.75):end,posCol)));
    % Are any position count larger than 1.5 x maxPos? If so, find the last
    % value where this is true
    if any(abs(d(:,posCol))>maxPos*1.5)
        trimInd=find(abs(d(:,posCol))>maxPos*1.5,1,'last')+1;
    else
        trimInd=1;
    end
    
    % Now trim to an even number of turbine rotations
    % NOTE! May only be effective for CCW rotating turbine
    [~,ind]=findpeaks(-abs(d(trimInd:end,posCol)-d(end,posCol)),'NPeaks',1);
    trimInd=trimInd+ind-1;
end
%% Process AMTI load cell data

if any(strcmp({HWconfig.inst},'AMTI top cell (digital)'))
    if isempty(dir(strcat('AMTIData_',num2str(timeID),'.bin')))
        error('No AMTIData file found with the given timeID num')
    end
    AMTIdata=readAMTIFile(strcat('AMTIData_',num2str(timeID),'.bin'));
    AMTIdata=AMTIdata(:,2:7); % remove first/last column
    % Make sure this and modelData are the same length (off by 1 is OK)
    if length(d)~=length(AMTIdata)
        if length(d)-length(AMTIdata)==1
            d(end,:)=[];
        elseif length(d)-length(AMTIdata)==-1
            AMTIdata(end,:)=[];
        else
            error('Aquisition rate or duration differs between AMTI and model')
        end
    end
    % Trim if requested
    if trim
        AMTIdata=AMTIdata(trimInd:end,:);
    end
    % Apply tare
    AMTIdata=bsxfun(@minus,AMTIdata,tare.AMTIforce);
    % Make coordinate system correct (need to verify thrust and lateral
    % force directions)
    
    % Apply transform from Alignment tare, if available
    if exist('align','var') %&& isfield(align,'AMTIalign')
        % Apply to Fx and Fy

        AMTIdata(:,1:2)=AMTIdata(:,1:2)*[align.AMTIalign]';
        % Apply to Mx and Mz
        AMTIdata(:,4:5)=AMTIdata(:,4:5)*[align.AMTIalign]';
        % For no bottom cell, assign forces here
        if ~any(strcmp({HWconfig.inst},'ATI bottom cell'))
            td.FX=AMTIdata(:,1);
            td.MX=AMTIdata(:,4);
            td.FY=AMTIdata(:,2);
            td.MY=AMTIdata(:,5);
            td.MZ=AMTIdata(:,6);
            td.FZ=AMTIdata(:,3);
        else
            td.AMTIdata=AMTIdata;
        end
    else
        % For no bottom cell, assign turbine forces
        if ~any(strcmp({HWconfig.inst},'ATI bottom cell'))
            td.FX=(-1).*AMTIdata(:,2);
            td.MX=AMTIdata(:,4);
            td.FY=AMTIdata(:,1);
            td.MY=(-1).*AMTIdata(:,5);
            td.MZ=AMTIdata(:,6);
            td.FZ=AMTIdata(:,3);
        else
            td.AMTIdata=AMTIdata;
        end
    end
    clear AMTIdata
end

%% Trim modelData if requested

if trim==1
    d=d(trimInd:end,:);
end

%% Process Lower ATI Loadcell Data

if any(strcmp({HWconfig.inst},'ATI bottom cell'))
    instInd=find(strcmp({HWconfig.inst},'ATI bottom cell'));
    ATIcellData=d(:,HWconfig(instInd).channels+1);
    if ~any(strcmp({HWconfig.inst},'AMTI top cell (digital)'))
        error('Bottom load cell only? Ben does not understand...')
    end
    % Apply Tare
    ATIcellData=bsxfun(@minus,ATIcellData,tare.DAQanalog(HWconfig(instInd).channels+1));
    
    % Apply transformation (to Newtons and real-world coordinates)
    ATIcellData=convertATI(ATIcellData);
    
    % Apply loadcell alignment to X and Y forces
    if exist('align','var') && isfield(align,'ATIalign')
        ATIcellData(:,1:2)=ATIcellData(:,1:2)*[align.ATIalign]';
        td.FX=td.AMTIdata(:,1)+ATIcellData(:,1);
        td.FY=td.AMTIdata(:,2)+ATIcellData(:,2);
    else
        warning('No force alignment tare found. Drag/lateral forces cannot be calculated')
    end
    % Add the parasitic bearing friction back in to total moment
    td.MZ=td.AMTIdata(:,6)+ATIcellData(:,6);
    % sum the vertical forces
    td.FZ=td.AMTIdata(:,3)+ATIcellData(:,3);
    % Save isolated lower cell forces
    td.ATIdata=ATIcellData;
    clear ATIcellData;
end
%% Process turbine position/vel./acc.

% Convert position to radians
turbPos=-d(:,posCol).*2*pi/(2^16);
% Apply offset from tare/blade mount position
turbPos=mod(unwrap(turbPos)+tare.posoffset,2*pi);
% Maybe unnecessary?
while any(turbPos<0) || any(turbPos>=2*pi)
    turbPos(turbPos>=2*pi)=turbPos(turbPos>=2*pi)-2*pi;
    turbPos(turbPos<0)=turbPos(turbPos<0)+2*pi;
end
td.turbPos=turbPos; clear turbPos;

% Calculate vel. and acc.
[td.turbVel,td.turbAcc]=multi_poly_diff(unwrap(td.turbPos),20,2);
td.turbVel=td.turbVel.*runInfo.frequency;
td.turbAcc=td.turbAcc.*runInfo.frequency.^2;

%% Process command/target signals

td.commandVel=d(:,posCol+1).*5.2358; % command velocity
td.commandTorque=0.332 .* d(:,posCol+2) - .0604; % command torque   Check this callibration! %%%%%%%

%% Process AMTI Top Cell Analog Torque Input

if any(strcmp({HWconfig.inst},'AMTI top cell torque (analog)'))
    instInd=find(strcmp({HWconfig.inst},'AMTI top cell torque (analog)'));
    AMTIanalagTorque=d(:,HWconfig(instInd).channels+1);
    % Apply tare
    AMTIanalagTorque=AMTIanalagTorque-tare.DAQanalog(HWconfig(instInd).channels+1);
    % Convert to N-m
    td.AMTIanalogTorque=AMTIanalagTorque;
    clear AMTIanalagTorque
end

%% Process Temperature Probe data

if any(strcmp({HWconfig.inst},'Temperature probe'))
    instInd=find(strcmp({HWconfig.inst},'Temperature probe'));
    temperature=d(:,HWconfig(instInd).channels+1);
    temperature=temperature.*51.5177 - 76.2556; % this callibration need to be checked with new wiring %%%%%%%%%%%%%%%%%%%%%
    td.temp=temperature;
    clear temperature
end

%% Process Vector Analog input signals

if any(strcmp({HWconfig.inst},'Vector (analog)'))
    instInd=find(strcmp({HWconfig.inst},'Vector (analog)'));
    td.analogVector=(d(:,HWconfig(instInd).channels+1)-2.5);%./2.5;
end

%% Process Vector Digital file

if any(strcmp({HWconfig.inst},'Vector (digital)'))
    startTime=runInfo.initTime;
    % Add the time to trim to starTime
    if trim
        startTime=startTime+trimInd/runInfo.frequency/(3600*24);
    end
    % End time. d has already been trimmed
    endTime=startTime+(length(d)/runInfo.frequency)/(3600*24);
    
    % if Vector Binary has not been processed, do so. (also, look in the
    % next folder up for it)
    if isempty(dir('vector.dat'))
        curdir=pwd;
        cd('../')
        if isempty(dir('vector.dat'))
            error('Can''t find the location of the vector.dat file')
        end
        if isempty(dir('Vector.mat'))
            processVectorBinary([],dspkVector);
        end
        load('Vector.mat');
        cd(curdir)
    else
        if isempty(dir('Vector.mat'))
            processVectorBinary([],dspkVector);
        end
        load('Vector.mat');
    end
    td.digitalVector=getVectorRange(startTime,endTime,Vector);
end

%% Vectrino processing
if any(strcmp({HWconfig.inst},'Vectrino (digital)'))
    [td.v0,td.v1,vectrinoRate]=vectrinoCalcPoint(timeID);
    
    % Make index vector (indicates which model data point each vectrino
    % data point goes with). Assumes the very first datapoints are aligned
    sampleMultiple=runInfo.frequency/vectrinoRate;
    if mod(sampleMultiple,1)~=0
        warning('Model frequency is not a multiple of vectrino frequency. This may cause issues.')
    end
    sInd=1:sampleMultiple:numDataPoints;
    
    % trim, if needed
    if trim
        VtrimInd=ceil((trimInd-1)/sampleMultiple+1);
        sInd=sInd-trimInd+1;
        sInd(sInd<1)=[];
        
        td.v0.U=td.v0.U(VtrimInd:end,:);
        td.v1.U=td.v1.U(VtrimInd:end,:);
        td.v0.badInds=td.v0.badInds(VtrimInd:end);
        td.v1.badInds=td.v1.badInds(VtrimInd:end);
        
        if length(sInd)~=length(td.v0.U(:,1))
            error('Error in computing Vectrino data indicie vector')
        end
    end
    td.v0.sInd=sInd;
    td.v1.sInd=sInd;
end

%% If given the moment of interia, we can calculate the fluid torque
if exist('MomentOfI','var') && ~isempty(MomentOfI)
    if ~strcmp(class(MomentOfI),'double')
        % Then it is a func model of some sort, not moment of inertia
        % get a finer turbine velocity (need to use this when model is
        % generated as well)
        [dx,~]=multi_poly_diff(unwrap(td.turbPos),3,2);
        dx=dx.*runInfo.frequency;
        % pad the fromt so it doesn't do (as much) weird shit
        dx=[ones(500,1).*dx(1);dx];
        t=0:0.001:(length(dx)-1)/runInfo.frequency;
        [y,~,~]=lsim(MomentOfI,dx,t);
        td.MZF=td.MZ-y(501:end);
    else
        td.MZF=td.MZ+td.turbAcc.*MomentOfI;
    end
end


%% Free suurf. transducer - still need to write
if any(strcmp({HWconfig.inst},'Free surface height transducer'))
    warning('Free surface transducer processing code not yet incorporated')
end


end
