function [U,Range,badInds]=vectrinoDataScrub(Data,Config,opts)
% Post processing code for Vectrino ADV velocity profiles

% This script removes points with "bad" correlation, either every point
% with correlation below 70% or the 10% of points with the worst
% correlation, whichever results in the removal of the fewest points

% In addition, despiking is performed on the beam velocities.  However,
% a high pass filter applied prior to despiking, reducing variance due to
% slow velocity oscillations and incresing the effectiveness of the
% despiking algorithm.

% Inputs: Data and Config are the structs generated by conversion of .ntk
% vectrino files to MATLAB format using the Vectrino software.  Velocity
% can be measured in either beam (preffered) or xyz coordinates
%
% opts =
%   0: (default) replaces bad points with NaN
%   1: Removes bad points
%   2: Replaces bad points with polynominal interpolation via neighbors

% Outputs:
% U - U is a cell array of length n, where n is the number of profiler bins.
% Each cell is a t by 4 array, where t is the number of time sample.
% The second dimension is the velocity % components, u_x, u_y, u_z1, u_z2
%
% Range - 1 x n vector, array of z distances from transducer head to sample
% position
%
% badInds - logical array, t x n, indicating where "bad" data points were
% found


if ~exist('opts','var') || isempty(opts)
    opts=0;
end
%% If needed, convert back to beam coordinates
Range = Data.Profiles_Range;

% High pass filter, for use later
    sf=Config.sampleRate;
    cutoffFreq = 3; % High pass filter cuttoff in Hz. May need to adjust
    dif = designfilt('highpassfir', 'FilterOrder', 20, 'CutoffFrequency', cutoffFreq, 'SampleRate', sf);

for j = 1:length(Range)
    % Get the tranform matrix
    T = reshape(Config.ProbeCalibration_calibrationMatrix(j,:),4,4)';
    
    % Get the velocity, transform if needed
    if isfield(Data,'Profiles_VelX')
        Uin=[Data.Profiles_VelX(:,j)';Data.Profiles_VelY(:,j)';...
            Data.Profiles_VelZ1(:,j)';Data.Profiles_VelZ2(:,j)'];
        Ubeam=(T\Uin)';
    else
        Ubeam=[Data.Profiles_VelBeam1(:,j),Data.Profiles_VelBeam2(:,j),...
            Data.Profiles_VelBeam3(:,j),Data.Profiles_VelBeam4(:,j)];
    end
    
    %% Find bad points based on correlation
    % NOTE: If a point is bad in one beam, all beams are considered bad.
    cor=[Data.Profiles_CorBeam1(:,j), Data.Profiles_CorBeam2(:,j), Data.Profiles_CorBeam3(:,j), Data.Profiles_CorBeam4(:,j)];
    corLogic=ones(size(cor));
    corLogic(cor<70)=0;
    corLogic2=prod(corLogic,2);
    badCor=zeros(length(corLogic2),1);
    if sum(corLogic2==0)<0.1*length(corLogic2)
        % Then chuck the ones less than 70
        badCor(corLogic2==0,:)=1;
    else
        % Chuck the worst 10%
        [~,inds]=sort(cor);
        inds=reshape(inds',numel(inds),1);
        uniqueInds=unique(inds,'stable');
        chuckInds=uniqueInds(1:round(length(corLogic2)*0.1));
        badCor(chuckInds)=1;
    end
    
        badSpike=zeros(length(corLogic2),1);
    %% Find bad points based on despike
    for i=1:4
        f=filtfilt(dif,detrend(double(Ubeam(:,i))));
        [~,spikeInds]=func_despike_phasespace3d(f);
        badSpike(spikeInds)=1;
    end
    
    %% Merge correlation and despike
    badInds=zeros(size(badSpike));
    badInds(badSpike | badCor)=1;
    badInds=logical(badInds);
    %% If needed, interpolate/remove points. Otherwise NaN
    
    if opts==1
        Ubeam(badInds,:)=[];
    elseif opts==0
        Ubeam(badInds,:)=NaN;
    elseif opts==2
        if badInds(1)
            % first good index
            fgind = find(~badInds,1,'first')+1;
            Ubeam(1,:) = Ubeam(fgind,:);
            badInds(1) = 0;
        end
        if badInds(end)
            % last good index
            bgind = find(~badInds,1,'last')+1;
            Ubeam(end,:) = Ubeam(bgind,:);
            badInds(end) = 0;
        end
        x=find(~badInds);
        xq=find(badInds);
        for i=1:4
            Ubeam(xq,i)=interp1(x,Ubeam(x,i),xq,'PCHIP');
        end
    end
    
    %% Transform back to XYZ coords
    U{j} = (T*Ubeam')';
end
end